﻿using log4net;
using Mono.Addins;
using Nini.Config;
using OpenMetaverse;
using OpenSim.Modules.DataValue.Storage;
using OpenSim.Region.Framework.Interfaces;
using OpenSim.Region.Framework.Scenes;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Security.Cryptography;
using System.Text;
using System.Timers;

[assembly: Addin("DataValueModule", "0.1")]
[assembly: AddinDependency("OpenSim.Region.Framework", OpenSim.VersionInfo.VersionNumber)]
namespace OpenSim.Modules.DataValue
{
    [Extension(Path = "/OpenSim/RegionModules", NodeName = "RegionModule", Id = "DataValueModule")]

    public class DataValue : INonSharedRegionModule
    {
        #region Region Module
        private static readonly ILog m_log = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

        private Scene m_scene = null;
        private IConfig m_config = null;
        private IScriptModuleComms m_scriptModule;

        private String m_storageTyp = null;
        private iStorage m_storage = null;

        public string Name
        {
            get { return "DataValueModule"; }
        }

        public Type ReplaceableInterface
        {
            get { return null; }
        }

        public void AddRegion(Scene scene)
        {

        }

        public void Close()
        {

        }

        public void Initialise(IConfigSource source)
        {
            try
            {
                m_config = source.Configs["XEngine"];

                m_storageTyp = m_config.GetString("DataStorageTyp", "RegionExtras").ToUpper().Trim();
            }
            catch (Exception e)
            {
                m_log.ErrorFormat("[" + Name + "]: initialization error: {0}", e.Message);
                return;
            }
        }

        public void RegionLoaded(Scene scene)
        {
            m_log.Info("[" + Name + "]: Load region " + scene.Name);

            m_scene = scene;

            if (m_storageTyp == "REGIONEXTRAS")
                m_storage = new RegionExtras(m_scene, m_config);

            if (m_storageTyp == "FILESYSTEM")
                m_storage = new FileSystem(m_scene, m_config);

            if (m_storageTyp == "MYSQL")
                m_storage = new MySQL(m_scene, m_config);

            if(m_storage == null)
                m_storage = new RegionExtras(m_scene, m_config);

            m_scriptModule = m_scene.RequestModuleInterface<IScriptModuleComms>();
            if (m_scriptModule == null)
            {
                m_log.ErrorFormat("[" + Name + "]: Failed to load IScriptModuleComms!");
                return;
            }

            try
            {
                m_scriptModule.RegisterScriptInvocation(this, "osGetDataValue");
                m_scriptModule.RegisterScriptInvocation(this, "osSetDataValue");
                m_scriptModule.RegisterScriptInvocation(this, "osDeleteDataValue");
                m_scriptModule.RegisterScriptInvocation(this, "osCheckDataValue");
            }
            catch (Exception e)
            {
                m_log.WarnFormat("[" + Name + "]: script method registration failed; {0}", e.Message);
            }
        }

        public void RemoveRegion(Scene scene)
        {
            
        }

        #endregion

        #region Script Funktions

        [ScriptInvocation]
        public string osGetDataValue(UUID hostID, UUID scriptID, string key)
        {

        }

        [ScriptInvocation]
        public void osSetDataValue(UUID hostID, UUID scriptID, string key, string value)
        {


        [ScriptInvocation]
        public void osDeleteDataValue(UUID hostID, UUID scriptID, string key, string value)
        {

        }

        [ScriptInvocation]
        public int osCheckDataValue(UUID hostID, UUID scriptID, string key)
        {

        }

        #endregion

    }
}
